// MachPCBDrill, NC drill/routing program for Mach3
// Copyright (C) 201, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "stdafx.h"
#include "PCBDrill.h"
#include "PCBDrillDlg.h"
#include "Xmlutil.h"
#include "GCode.h"
#include "DrillSetup.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


// CPCBDrillDlg dialog
const short OEMDRO_X_AXIS		= 800;
const short OEMDRO_Y_AXIS		= 801;
const short OEMDRO_Z_AXIS		= 802;

static const CSize szMinimumSize( 473, 615 );
//const char *sConfigFile = "d:\\Projects\\Gecko\\PCBDrill\\Config.xml";
const char *sConfigFile = "Config.xml";

#define TIMER_ID_POLL_MACH		33
#define POLL_MACH_PERIOD		125			// 125ms, 8Hz


CPCBDrillDlg::CPCBDrillDlg(CWnd* pParent /*=NULL*/)
    : CDialog(CPCBDrillDlg::IDD, pParent),
    m_drillfile(NULL),
    m_bResized(true),
    m_HighlightCoord(NULL),
    m_SelectedCoord(NULL),
    m_DrillCoord(NULL),
    m_bFlip( false ),
    m_dAngle( 0.0 ),
    m_dClearance( 10.0 ),
    m_dDrillDepth( -2.6 ),
    m_bRouteStep( false ),
    m_dRouteStep( 0 ),
    m_dPlungeSpeed( 120 ),
    m_dRouteSpeed( 450 ),
    m_bCannedDrill( false ),
    m_sCannedCode( "G81" ),
    m_sInitString(""),
    m_bMetric( true ),
    m_bDrill( true ),
    m_bRoute(true )
{
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

CPCBDrillDlg::~CPCBDrillDlg()
{
    if ( m_drillfile != NULL )
	delete m_drillfile;
    m_drillfile = NULL;
}

void AFXAPI DDX_CheckBool( CDataExchange* pDX, int nIDC, bool& value )
{
    if ( pDX->m_bSaveAndValidate )
    {
	int n;
	DDX_Check(pDX,nIDC,n);
	if ( n == BST_CHECKED )
	    value = true;
	else
	    value = false;
    }
    else
    {
	int n = value ? BST_CHECKED : BST_UNCHECKED;
	DDX_Check(pDX,nIDC,n);
    }
}


void AFXAPI DDX_RadioBool( CDataExchange* pDX, int nIDC, bool& value )
{
    if ( pDX->m_bSaveAndValidate )
    {
	int n;
	DDX_Radio(pDX,nIDC,n);
	if ( n == 0 )
	    value = true;
	else
	    value = false;
    }
    else
    {
	int n = value ? 0 : 1;
	DDX_Radio(pDX,nIDC,n);
    }
}


void CPCBDrillDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    DDX_Control(pDX, IDC_EDIT_LOG, m_txtLog);
    DDX_Control(pDX, IDC_CHECK_ROUTE, m_chkRoute);
    DDX_Control(pDX, IDC_CHECK_DRILL, m_chkDrill);
    DDX_Control(pDX, IDC_MACH_INDICATOR, m_staticIndicator);
    DDX_Control(pDX, IDC_STATIC_MACHX, m_lblMachX);
    DDX_Control(pDX, IDC_STATIC_MACHY, m_lblMachY);
    DDX_Control(pDX, IDC_STATIC_MACHZ, m_lblMachZ);
    DDX_Control(pDX, IDC_CHECK_ROUTE_STEP, m_chkRouteStep);
    DDX_Control(pDX, IDC_CHECK_DRILL_CYCLE, m_chkDrillCycle);
    DDX_Control(pDX, IDC_RADIO_MM, m_btnMM);
    DDX_Control(pDX, IDC_RADIO_IN, m_btnIN);
    DDX_Control(pDX, IDC_EDIT_CLEARANCE, m_txtClearance);
    DDX_Control(pDX, IDC_EDIT_ROUTE_STEP, m_txtRouteStep);
    DDX_Control(pDX, IDC_EDIT_PLUNGE_SPEED, m_txtPlungeSpeed);
    DDX_Control(pDX, IDC_EDIT_ROUTE_SPEED, m_txtRouteSpeed);
    DDX_Control(pDX, IDC_EDIT_DRILL_CYCLE, m_txtDrillCycle);

    DDX_Text(pDX, IDC_EDIT_CLEARANCE, m_dClearance );
    DDX_Text(pDX, IDC_EDIT_DRILL_DEPTH, m_dDrillDepth );
    DDX_Text(pDX, IDC_EDIT_ROUTE_STEP, m_dRouteStep );
    DDX_Text(pDX, IDC_EDIT_PLUNGE_SPEED, m_dPlungeSpeed );
    DDX_Text(pDX, IDC_EDIT_ROUTE_SPEED, m_dRouteSpeed );
    DDX_Text(pDX, IDC_EDIT_DRILL_CYCLE, m_sCannedCode );
    DDX_CheckBool( pDX, IDC_CHECK_DRILL_CYCLE, m_bCannedDrill );
    DDX_RadioBool( pDX, IDC_RADIO_MM, m_bMetric );
    DDX_CheckBool( pDX, IDC_CHECK_ROUTE_STEP, m_bRouteStep );
    DDX_CheckBool( pDX, IDC_CHECK_ROUTE, m_bRoute );
    DDX_CheckBool( pDX, IDC_CHECK_DRILL, m_bDrill );

    if ( !pDX->m_bSaveAndValidate )
    {
	EnableControls();
    }
}

BEGIN_MESSAGE_MAP(CPCBDrillDlg, CDialog)
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    ON_WM_SIZE()
    ON_WM_GETMINMAXINFO()
    //}}AFX_MSG_MAP
    ON_COMMAND(ID_FILE_OPEN32771, &CPCBDrillDlg::OnFileOpen)
    ON_COMMAND(ID_FILE_EXIT, &CPCBDrillDlg::OnFileExit)
    ON_COMMAND(ID_FILE_PREFERENCES, &CPCBDrillDlg::OnFilePreferences)
    ON_WM_MOUSEMOVE()
    ON_WM_LBUTTONDOWN()
    ON_BN_CLICKED(IDC_BUTTON_LOCATE, &CPCBDrillDlg::OnBnClickedButtonLocate)
    ON_BN_CLICKED(IDC_BUTTON_GOTO, &CPCBDrillDlg::OnBnClickedButtonGoto)
    ON_WM_MOVE()
    ON_WM_SHOWWINDOW()
    ON_BN_CLICKED(IDC_CHECK_ROUTE, &CPCBDrillDlg::OnBnClickedCheckRoute)
    ON_BN_CLICKED(IDC_CHECK_DRILL, &CPCBDrillDlg::OnBnClickedCheckDrill)
    ON_WM_TIMER()
    ON_BN_CLICKED(IDC_BUTTON_ROTATE, &CPCBDrillDlg::OnBnClickedButtonRotate)
    ON_BN_CLICKED(IDC_BUTTON_FLIP, &CPCBDrillDlg::OnBnClickedButtonFlip)
    ON_BN_CLICKED(IDC_CHECK_DRILL_CYCLE, &CPCBDrillDlg::OnBnClickedCheckDrillCycle)
    ON_BN_CLICKED(IDC_CHECK_ROUTE_STEP, &CPCBDrillDlg::OnBnClickedCheckRouteStep)
    ON_BN_CLICKED(IDC_BUTTON_GCODE, &CPCBDrillDlg::OnBnClickedButtonGcode)
    ON_BN_CLICKED(IDC_BUTTON_DELETE, &CPCBDrillDlg::OnBnClickedButtonDelete)
END_MESSAGE_MAP()


// CPCBDrillDlg message handlers

BOOL CPCBDrillDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Set the icon for this dialog.  The framework does this automatically
    //  when the application's main window is not a dialog
    SetIcon(m_hIcon, TRUE);			// Set big icon
    SetIcon(m_hIcon, FALSE);		// Set small icon

    m_statusBar.Create( WS_CHILD | WS_VISIBLE | SBARS_SIZEGRIP, CRect(0,0,0,0), this, 0 );
    ResizeWindow();

    // Read user settings

    LoadConfigFile( sConfigFile );
    UpdateData( false );

    m_penLocated.CreatePen(PS_SOLID,2, RGB(0,0,0) );
    m_penMove.CreatePen(PS_DOT,1, RGB(128,128,128) );
    m_penMoveHighlight.CreatePen(PS_DOT,1, RGB(255,255,255) );
    m_penRoute.CreatePen(PS_SOLID,2, RGB(0,0,0) );
    m_penRouteHighlight.CreatePen(PS_SOLID,2, RGB(255,255,255) );

    m_brBkgnd.CreateSolidBrush( RGB(255,0,0) );

    m_chkDrill.SetCheck( BST_CHECKED );
    m_chkRoute.SetCheck( BST_CHECKED );

    EnableControls();

    SetTimer( TIMER_ID_POLL_MACH, POLL_MACH_PERIOD, NULL );

    return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CPCBDrillDlg::OnPaint()
{
    if (IsIconic())
    {
	CPaintDC dc(this); // device context for painting

	SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

	// Center icon in client rectangle
	int cxIcon = GetSystemMetrics(SM_CXICON);
	int cyIcon = GetSystemMetrics(SM_CYICON);
	CRect rect;
	GetClientRect(&rect);
	int x = (rect.Width() - cxIcon + 1) / 2;
	int y = (rect.Height() - cyIcon + 1) / 2;

	// Draw the icon
	dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
	// Paint the PCB
	CPaintDC dc(this);
	dc.FillSolidRect( m_rcPCBArea, RGB(255,255,255) );


	if ( m_drillfile != NULL )
	{
	    bool bDrill = m_chkDrill.GetCheck() == BST_CHECKED;
	    bool bRoute = m_chkRoute.GetCheck() == BST_CHECKED;

	    int nPadSize = (int)(1.5 * m_dScale + 0.5);
	    if ( nPadSize < 2 )
		nPadSize = 2;

	    if ( m_bResized )
	    {
		RecomputeScreenCoordinates( nPadSize );
		m_bResized = false;
	    }

	    CRect rcBounds(m_nXOffset - nPadSize, m_nYOffset + nPadSize, (int)(m_drillfile->Width() * m_dScale + m_nXOffset) + nPadSize, (int)(-m_drillfile->Height() * m_dScale + m_nYOffset) - nPadSize );
	    dc.FillSolidRect( rcBounds, RGB(200,230,20) );

	    CPen *pOldPen = dc.SelectObject( &m_penLocated );

	    int nLastX = m_drillfile->Coords.front()->DisplayX;
	    int nLastY = m_drillfile->Coords.front()->DisplayY;
	    bool bDrilled = false;
	    std::list<NCCoord*>::iterator it;
	    for ( it = m_drillfile->Coords.begin(); it != m_drillfile->Coords.end(); it++ )
	    {
		NCCoord *c = *it;
		NCRoute *r = dynamic_cast<NCRoute*>(*it);

		if ( r == NULL && bDrill )
		{
		    // Drill
		    if ( bDrilled )
			dc.FillSolidRect( c->DisplayRect, RGB(255,255,255) );
		    else if ( m_SelectedCoord == c )
			dc.FillSolidRect( c->DisplayRect, RGB(0,0,0) );
		    else if ( m_HighlightCoord == c )
			dc.FillSolidRect( c->DisplayRect, RGB(191,191,0) );
		    else
			dc.FillSolidRect( c->DisplayRect, RGB(255,0,0) );

		    if ( bDrilled )
			dc.SelectObject( &m_penMoveHighlight );
		    else
			dc.SelectObject( &m_penMove );

		    dc.MoveTo( nLastX, nLastY );
		    dc.LineTo( c->DisplayX, c->DisplayY );				
		}
		else if ( r != NULL && bRoute )
		{ 
		    // route
		    if (!r->Plunged)
			if ( bDrilled )
			    dc.SelectObject( &m_penMoveHighlight );
			else
			    dc.SelectObject( &m_penMove );
		    else
			if ( bDrilled )
			    dc.SelectObject( &m_penRouteHighlight );
			else
			    dc.SelectObject( &m_penRoute );

		    dc.MoveTo( nLastX, nLastY );
		    dc.LineTo( r->DisplayX, r->DisplayY );				
		}

		if ( c == m_DrillCoord )
		    bDrilled = false;

		nLastX = c->DisplayX;
		nLastY = c->DisplayY;
	    }

	    LocatedCoordCollection::iterator itLocated;
	    for ( itLocated = m_LocatedCoords.begin(); itLocated != m_LocatedCoords.end(); itLocated++ )
	    {
		CRect rc = (*itLocated).m_coord->DisplayRect;
		rc.InflateRect(1,1,0,0);
		dc.MoveTo( rc.left, rc.top );
		dc.LineTo( rc.left, rc.bottom );
		dc.LineTo( rc.right, rc.bottom );
		dc.LineTo( rc.right, rc.top );
		dc.LineTo( rc.left, rc.top );
	    }

	    dc.SelectObject( pOldPen );

	}
	m_bResized = true;
    }
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CPCBDrillDlg::OnQueryDragIcon()
{
    return static_cast<HCURSOR>(m_hIcon);
}

bool Near( double a, double b )
{
    if ( abs(a-b) < 0.1 )
	return true;
    else 
	return false;
}

void CPCBDrillDlg::OnFileOpen()
{
    //#define DEBUG_FILE
    CFileDialog fileOpen( TRUE, "*.txt", NULL, OFN_ENABLESIZING | OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST, "NC drill files (*.txt)|*.txt|All files (*.*)|*.*|", this );
    if ( fileOpen.DoModal() == IDOK )
    {
	m_sFileName = fileOpen.GetPathName();

	m_drillfile = new CNCDrillFile();
	m_drillfile->Load(m_sFileName);

	theApp.LogMessage( "Loaded NC File '%s'", (LPCSTR)m_sFileName );
	theApp.LogMessage( "    %d tools, %d holes, %d routes", m_drillfile->Tools.size(), m_drillfile->CoordCount(), m_drillfile->RouteCount() );
	m_bFlip = false;
	m_dAngle = 0.0;

	CalculateScale();
	InvalidateRect( m_rcPCBArea, false );
	EnableControls();
	// create a default 1 to 1 transformation.
	m_TransformMatrix = CMatrix( 1, 0, 0, 1, 0, 0 );


#ifdef DEBUG_FILE
	std::list<NCCoord*>::iterator it;
	for ( it = m_drillfile->Coords.begin(); it != m_drillfile->Coords.end(); it++ )
	{
	    NCCoord *c = *it;

	    if ( Near(c->X ,48.0) && Near(c->Y,18.5) )
	    {
		//40	18.5	37.312	22.003
		CLocatedCoord lc( c, 37.312, 22.003 );
		m_LocatedCoords.push_back( lc );
	    }
	    else if ( Near(c->X,48.0) && Near(c->Y, 120.0) )
	    {
		//40	120	40.854	123.442
		CLocatedCoord lc( c, 40.854, 123.442 );
		m_LocatedCoords.push_back( lc );
	    }
	    else if ( Near(c->X,132.195) && Near(c->Y,103.0192) )
	    {
		//124.195	103.0192	84.76	90.545
		CLocatedCoord lc( c, 84.76, 90.545 );
		m_LocatedCoords.push_back( lc );
	    }
	}
	if ( m_LocatedCoords.size() == 3 )
	    CalculateTransformation();
#endif

    }

}

void CPCBDrillDlg::OnFileExit()
{
    PostMessage(WM_COMMAND, IDOK, 0L);
}

void CPCBDrillDlg::OnFilePreferences()
{
    CDrillSetupDlg dlg(this);

    dlg.SetInitString( m_sInitString );
    if ( dlg.DoModal() == IDOK )
    {
	m_sInitString = dlg.GetInitString();
    }
}


void CPCBDrillDlg::OnSize( UINT nType, int cx, int cy )
{
    if ( IsWindowVisible() )
    {
	ResizeWindow();
    }
}

void CPCBDrillDlg::OnGetMinMaxInfo( MINMAXINFO* lpMMI  )
{
    lpMMI->ptMinTrackSize.x = szMinimumSize.cx;
    lpMMI->ptMinTrackSize.y = szMinimumSize.cy;
}


void CPCBDrillDlg::ResizeWindow()
{
    const int SPACING = 4;

    CRect rc;
    GetClientRect( rc );

    // Resize the rest
    CRect rcStatusBar;
    m_statusBar.MoveWindow(0,0,0,0);
    m_statusBar.GetWindowRect( rcStatusBar );

    // log window
    CRect rcLog;
    m_txtLog.GetWindowRect( rcLog );
    rcLog.MoveToY( rc.Height() - rcStatusBar.Height() - rcLog.Height() - SPACING );
    rcLog.left = SPACING;
    rcLog.right = rc.Width() - SPACING;
    m_txtLog.MoveWindow( rcLog );

    // PCB display area
    m_rcPCBArea.left = rc.left + SPACING;
    m_rcPCBArea.right = rc.right - SPACING;

    GetWindowRect( rc );
    rc.top += ::GetSystemMetrics(SM_CYBORDER) + ::GetSystemMetrics(SM_CYCAPTION) + ::GetSystemMetrics(SM_CYMENU);
    CRect rcBtnFlip;
    GetDlgItem(IDC_BUTTON_FLIP)->GetWindowRect( rcBtnFlip );
    m_rcPCBArea.top = rcBtnFlip.bottom - rc.top + 2 * SPACING;
    m_rcPCBArea.bottom = rcLog.top - SPACING;

    rc.top = m_rcPCBArea.top;
    //InvalidateRect( rc );

    CalculateScale();
    Invalidate();
}


void CPCBDrillDlg::CalculateScale()
{
    if ( m_drillfile != NULL )
    {
	double dXScale = ((double)m_rcPCBArea.Width())/m_drillfile->Width();
	double dYScale = ((double)m_rcPCBArea.Height())/m_drillfile->Height();

	if ( dXScale < dYScale )
	    m_dScale = dXScale;
	else
	    m_dScale = dYScale;
	m_dScale *= 0.9;

	m_nXOffset = m_rcPCBArea.left + (m_rcPCBArea.Width() - (int)(m_drillfile->Width() * m_dScale)) / 2;
	m_nYOffset = (int)(m_drillfile->Height() * m_dScale) + m_rcPCBArea.top + (m_rcPCBArea.Height() - (int)(m_drillfile->Height() * m_dScale)) / 2;

	m_bResized = true;
    }
}

void CPCBDrillDlg::LogMessage( LPCSTR sFormat, va_list args )
{
    CString sMsg;
    sMsg.FormatV( sFormat, args );

    TRACE( sMsg );
    TRACE( "\r\n" );

    m_txtLog.SetSel( 0x7FFF, 0x7FFF, TRUE );
    m_txtLog.ReplaceSel( sMsg );
    m_txtLog.SetSel( 0x7FFF, 0x7FFF, TRUE );
    m_txtLog.ReplaceSel( "\r\n" );
}



void CPCBDrillDlg::LoadConfigFile( LPCSTR sFilename )
{
    MSXML2::IXMLDOMDocumentPtr pDoc;
    HRESULT hr = pDoc.CreateInstance( "MSXML.DOMDocument" );
    if ( FAILED(hr) )
    {
	theApp.LogMessage( "Failed to create IXMLDOMDocument. Reason Code: %08X", hr );
	return;
    }

    pDoc->async = false;
    if( pDoc->load((LPCSTR)sFilename) != VARIANT_TRUE )
    {
	MSXML2::IXMLDOMParseErrorPtr err = pDoc->parseError;
	theApp.LogMessage( "Failed to open configuration file '%s'", sFilename );
	theApp.LogMessage( "Reason: %s", (LPCSTR)(err->reason) );
	return;
    }

    const LPCSTR sAppConfigSection = "/configuration/AppConfig";
    MSXML2::IXMLDOMNodePtr pAppConfigNode = pDoc->selectSingleNode(sAppConfigSection);
    if ( pAppConfigNode != NULL )
    {
	LoadConfig( pAppConfigNode );
    }

}

void CPCBDrillDlg::LoadConfig( MSXML2::IXMLDOMNodePtr pNode )
{
    m_dClearance = CXmlUtil::GetAttributeDouble( pNode, "clearance", m_dClearance );
    m_dDrillDepth = CXmlUtil::GetAttributeDouble( pNode, "drillDepth", m_dDrillDepth );
    m_bRouteStep = CXmlUtil::GetAttributeBool( pNode, "routeStepping", m_bRouteStep );
    m_dRouteStep = CXmlUtil::GetAttributeDouble( pNode, "routeStep", m_dRouteStep );
    m_dPlungeSpeed = CXmlUtil::GetAttributeDouble( pNode, "plungeSpeed", m_dPlungeSpeed );
    m_dRouteSpeed = CXmlUtil::GetAttributeDouble( pNode, "routeSpeed", m_dRouteSpeed );
    m_bCannedDrill = CXmlUtil::GetAttributeBool( pNode, "cannedDrill", m_bCannedDrill );
    m_sCannedCode = CXmlUtil::GetAttributeString( pNode, "cannedCode", m_sCannedCode );
    m_bMetric = CXmlUtil::GetAttributeBool( pNode, "metric", m_bMetric );
    m_sInitString = CXmlUtil::GetAttributeString( pNode, "initString", m_sInitString );
}

void CPCBDrillDlg::OnMouseMove(UINT nFlags, CPoint point)
{
    if ( m_drillfile == NULL )
	return;

    if ( !m_rcPCBArea.PtInRect(point) )
	return;


    std::list<NCCoord*>::iterator it;
    for ( it = m_drillfile->Coords.begin(); it != m_drillfile->Coords.end(); it++ )
    {
	NCCoord *c = *it;

	if ( c->DisplayRect.PtInRect( point ) )
	{
	    if ( m_HighlightCoord != NULL )
		InvalidateRect( m_HighlightCoord->DisplayRect, false );
	    m_HighlightCoord = c;
	    InvalidateRect( m_HighlightCoord->DisplayRect, false );

	    CString sMsg;
	    sMsg.Format( "Line:%d X:%f Y:%f T:%d (%1.4f)", c->Line, c->X, c->Y, c->Tool->Id, c->Tool->Size );
	    m_statusBar.SetText( sMsg, 255, 0 );
	    return;
	}
    }
    if ( m_HighlightCoord != NULL )
	InvalidateRect( m_HighlightCoord->DisplayRect, false );
    m_HighlightCoord = NULL;
    m_statusBar.SetText( "", 255, 0 );
}

void CPCBDrillDlg::OnLButtonDown(UINT nFlags, CPoint point)
{
    if ( m_SelectedCoord != NULL )
	InvalidateRect( m_SelectedCoord->DisplayRect, false );

    m_SelectedCoord = m_HighlightCoord;

    if ( m_SelectedCoord != NULL )
	InvalidateRect( m_SelectedCoord->DisplayRect, false );
}


void CPCBDrillDlg::OnBnClickedButtonLocate()
{
    if ( m_SelectedCoord == NULL )
    {
	MessageBox( "No PCB Pad is selected" );
	return;
    }

    Mach4::IMyScriptObjectPtr pObj = GetMachPointer();
    if ( pObj == NULL )
	return;

    double dX = pObj->GetOEMDRO(OEMDRO_X_AXIS);
    double dY = pObj->GetOEMDRO(OEMDRO_Y_AXIS);

    // Check for replacements
    bool bReplaced = false;
    LocatedCoordCollection::iterator it;
    for ( it = m_LocatedCoords.begin(); it != m_LocatedCoords.end(); it++ )
    {
	CLocatedCoord &c = *it;

	if ( c.m_coord == m_SelectedCoord )
	{
	    c.m_x = dX;
	    c.m_y = dY;
	    bReplaced = true;
	    break;
	}
    }

    if ( !bReplaced )	// New point
    {
	if ( m_LocatedCoords.size() == 3 )
	{
	    MessageBox( "Only 3 points can be located.  Delete an existing point if necessary");
	}
	else
	{
	    CRect rc = m_SelectedCoord->DisplayRect;
	    rc.InflateRect(2,2);
	    InvalidateRect( rc, false );
	    CLocatedCoord c( m_SelectedCoord, dX, dY );
	    m_LocatedCoords.push_back( c );
	}
    }

    if ( m_LocatedCoords.size() == 3 )
	CalculateTransformation();
}

void CPCBDrillDlg::CalculateTransformation()
{
    CPointF dest[3];
    CPointF src[3];

    int i = 0;
    LocatedCoordCollection::iterator it;
    for ( it = m_LocatedCoords.begin(); it != m_LocatedCoords.end(); it++, i++ )
    {
	CLocatedCoord &c = *it;

	src[i].X = c.m_coord->X;
	src[i].Y = c.m_coord->Y;

	dest[i].X = c.m_x;
	dest[i].Y = c.m_y;
    }

    m_TransformMatrix = CTransformation::affineXformCoeffs(dest, src);
}


void CPCBDrillDlg::EnableControls()
{
    static UINT nIDs[] = {  IDC_BUTTON_LOCATE,
	IDC_BUTTON_GOTO,
	IDC_BUTTON_FLIP,
	IDC_BUTTON_ROTATE };
    for ( int i = 0; i < countof(nIDs); i++ )
	GetDlgItem(nIDs[i])->EnableWindow(m_drillfile != NULL);

    bool bOn = m_chkRouteStep.GetCheck() == BST_CHECKED;
    m_txtRouteStep.EnableWindow( bOn );

    bOn = m_chkDrillCycle.GetCheck() == BST_CHECKED;
    m_txtDrillCycle.EnableWindow( false );
}



void CPCBDrillDlg::OnOK()
{
    Close();
    __super::OnOK();
}


void CPCBDrillDlg::OnCancel()
{
    Close();
    __super::OnCancel();
}

void CPCBDrillDlg::Close()
{
    if ( !UpdateData( true ) )
	return;

    SaveConfig();
}


void CPCBDrillDlg::SaveConfig()
{
    // Build the XML data.
    MSXML2::IXMLDOMDocumentPtr pDoc;
    pDoc.CreateInstance( "MSXML.DOMDocument" );
    pDoc->async = false;
    if ( !pDoc->load( sConfigFile ) )
    {
	// No existing config file, create a new one.
	pDoc->appendChild( pDoc->createElement("configuration") );
    }

    MSXML2::IXMLDOMNodePtr pNewRootNode;
    MSXML2::IXMLDOMElementPtr pElem;

    pElem = pDoc->createElement("AppConfig");

    CXmlUtil::AddAttribute( pElem, "clearance", m_dClearance );
    CXmlUtil::AddAttribute( pElem, "drillDepth", m_dDrillDepth );
    CXmlUtil::AddAttribute( pElem, "routeStepping", m_bRouteStep );
    CXmlUtil::AddAttribute( pElem, "routeStep", m_dRouteStep );
    CXmlUtil::AddAttribute( pElem, "plungeSpeed", m_dPlungeSpeed );
    CXmlUtil::AddAttribute( pElem, "routeSpeed", m_dRouteSpeed );
    CXmlUtil::AddAttributeBool( pElem, "cannedDrill", m_bCannedDrill );
    CXmlUtil::AddAttribute( pElem, "cannedCode", m_sCannedCode );
    CXmlUtil::AddAttributeBool( pElem, "metric", m_bMetric );
    CXmlUtil::AddAttribute( pElem, "initString", m_sInitString );

    pNewRootNode = pElem;

    // If the root node is already there, replace it
    MSXML2::IXMLDOMNodePtr pConfigNode = pDoc->selectSingleNode("/configuration/AppConfig");
    if ( pConfigNode != NULL )
    {
	pConfigNode->parentNode->replaceChild( pNewRootNode, pConfigNode );
    }
    else
    {
	MSXML2::IXMLDOMNodePtr pRootNode = pDoc->documentElement;
	MSXML2::IXMLDOMNodePtr pNode = pRootNode->selectSingleNode("/configuration");

	pNode->appendChild( pNewRootNode );
    }

    pDoc->save( _variant_t(sConfigFile) );
}


class Recompute
{
private:
    int m_nPadSize;
    double m_dXmin;
    double m_dYmin;
    double m_dScale;
    int m_nXOffset;
    int m_nYOffset;

public:
    Recompute( int nPadSize, double dXmin, double dYmin, double dScale, int nXOffset, int nYOffset )
	: m_nPadSize( nPadSize ),
	m_dXmin( dXmin ),
	m_dYmin( dYmin ),
	m_dScale( dScale ),
	m_nXOffset(nXOffset ),
	m_nYOffset( nYOffset )
    {}

    void operator ( ) ( NCCoord *c ) const
    {
	int nX = (int)((c->X - m_dXmin) * m_dScale + m_nXOffset);
	int nY = (int)((m_dYmin -c->Y) * m_dScale + m_nYOffset);

	c->DisplayX = nX;
	c->DisplayY = nY;
	c->DisplayRect.left = nX - m_nPadSize / 2;
	c->DisplayRect.right = nX + m_nPadSize / 2;
	c->DisplayRect.top = nY - m_nPadSize / 2;
	c->DisplayRect.bottom = nY + m_nPadSize / 2;
    }
};


void CPCBDrillDlg::RecomputeScreenCoordinates( int nPadSize )
{
    std::for_each( m_drillfile->Coords.begin(), m_drillfile->Coords.end(), Recompute(nPadSize,m_drillfile->Xmin,m_drillfile->Ymin, m_dScale, m_nXOffset, m_nYOffset) );
}


void CPCBDrillDlg::OnBnClickedCheckRoute()
{
    InvalidateRect( m_rcPCBArea, false );
}

void CPCBDrillDlg::OnBnClickedCheckDrill()
{
    InvalidateRect( m_rcPCBArea, false );
}

void CPCBDrillDlg::OnBnClickedButtonGoto()
{
    if ( m_SelectedCoord != NULL && 
	( m_LocatedCoords.size() == 3 ||
	(m_LocatedCoords.size() == 0 && m_TransformMatrix.IsIdentity()) ) )
    {
	Mach4::IMyScriptObjectPtr pObj = GetMachPointer();
	if ( pObj != NULL )
	{
	    CPointF src( m_SelectedCoord->X, m_SelectedCoord->Y );
	    CPointF dest = m_TransformMatrix.TransformPoint( src );

	    CString sCode;
	    sCode.Format( "G0 X%.5f Y%.5f", dest.X, dest.Y );
	    pObj->Code( _bstr_t(sCode) );
	}
    }
}

void CPCBDrillDlg::SetConnectState( bool bConnected )
{
    //m_bG100Connected = bConnected;
    //m_JogDlg.SetConnectState( bConnected );
    //EnableControls();
    if ( bConnected )
	m_staticIndicator.SetBitmap( LoadBitmap( AfxGetInstanceHandle(),  MAKEINTRESOURCE(IDB_GREEN_LIGHT) ) );
    else
	m_staticIndicator.SetBitmap( LoadBitmap( AfxGetInstanceHandle(),  MAKEINTRESOURCE(IDB_RED_LIGHT) ) );
}

Mach4::IMyScriptObjectPtr CPCBDrillDlg::GetMachPointer()
{
    try
    {
	if ( m_pMach4 == NULL )
	{
	    if ( m_pMach4.GetActiveObject("Mach4.Document") == S_OK)
	    {
		IDispatchPtr ptr = m_pMach4->GetScriptDispatch();
		m_pScriptObj = ptr;

		// Fudge - Mach seems to get upset after the last release is released - ie we can't connect
		// after the release count goes to 0.  We add another ref, which keeps it alive
		m_pMach4->AddRef();
		m_pScriptObj->AddRef();

		SetConnectState( true );
	    }
	}
	else
	{
	    m_pScriptObj->GetOEMDRO(1);	// test if connection is still up
	}
    }
    //catch ( _com_exception e )
    catch (...)
    {
	SetConnectState( false );
	m_pMach4 = NULL;
	m_pScriptObj = NULL;
    }

    return m_pScriptObj;
}

void CPCBDrillDlg::OnTimer(UINT_PTR nIDEvent)
{
    if ( nIDEvent == TIMER_ID_POLL_MACH )
    {
	try
	{
	    Mach4::IMyScriptObjectPtr pObj = GetMachPointer();
	    if ( pObj != NULL )
	    {
		double dX = pObj->GetOEMDRO( OEMDRO_X_AXIS );
		double dY = pObj->GetOEMDRO( OEMDRO_Y_AXIS );
		double dZ = pObj->GetOEMDRO( OEMDRO_Z_AXIS );

		CString sX, sY, sZ;
		// if ( mm )
		sX.Format( "%.03f", dX );
		sY.Format( "%.03f", dY );
		sZ.Format( "%.03f", dZ );

		m_lblMachX.SetWindowText( sX );
		m_lblMachY.SetWindowText( sY );
		m_lblMachZ.SetWindowText( sZ );

	    }

	}
	//catch (_com_exception e)
	catch (...)
	{
	    m_pScriptObj = NULL;
	    m_pMach4 = NULL;

	    SetConnectState( false );
	}
    }

    CDialog::OnTimer(nIDEvent);
}

class Transform
{
private:
    double m_dAngle;
    double m_dCenterX, m_dCenterY;
    bool m_bFlip;

public:
    Transform( bool bFlip, double dAngle, double dX, double dY )
	: m_bFlip(bFlip),
	m_dAngle( dAngle ),
	m_dCenterX( dX ),
	m_dCenterY( dY )
    {}

    void operator ( ) ( NCCoord *c ) const
    {
	c->Transform( m_bFlip, m_dAngle, m_dCenterX, m_dCenterY );
    }
};


void CPCBDrillDlg::OnBnClickedButtonRotate()
{
    m_dAngle += 90.0;
    DoTransform();
}

void CPCBDrillDlg::DoTransform()
{
    std::for_each( m_drillfile->Coords.begin(), m_drillfile->Coords.end(), Transform( m_bFlip, m_dAngle,  (m_drillfile->Xmin + m_drillfile->Xmax)/2, (m_drillfile->Ymin + m_drillfile->Ymax)/2) );
    m_drillfile->CalcualteBoundingRect();
    CalculateScale();
    m_LocatedCoords.clear();
    InvalidateRect( m_rcPCBArea, false );
}


void CPCBDrillDlg::OnBnClickedButtonFlip()
{
    m_bFlip = !m_bFlip;
    DoTransform();
}

void CPCBDrillDlg::OnBnClickedCheckDrillCycle()
{
    EnableControls();
}

void CPCBDrillDlg::OnBnClickedCheckRouteStep()
{
    EnableControls();
}


CString CPCBDrillDlg::FormatNumber( double d )
{
    CString s;
    if ( m_bMetric )
    {
	if ( !m_drillfile->GetMetric() )
	    d /= 25.4;
    }
    else
    {
	if ( m_drillfile->GetMetric() )
	    d *= 25.4;
    }

    if ( m_bMetric )
	s.Format( "%.3f", d );
    else
	s.Format( "%.4f", d );

    return s;
}

void CPCBDrillDlg::OnBnClickedButtonGcode()
{
    if ( !UpdateData(true) )
	return;

    CGCode gc( m_bMetric );

    CString s;
    s += "GCode for PCB Drilling of file '";
    s += m_sFileName;
    s += "'";
    gc.SendComment( s );

    if ( m_sInitString.GetLength() != 0 )
	gc.Append( (const char *)m_sInitString );

    NCTool *pCurrentTool = NULL;
    bool bPlunged = false;

    // Group commands by tool.
    std::list<NCCoord*>::iterator it;
    for ( it = m_drillfile->Coords.begin(); it != m_drillfile->Coords.end(); it++ )
    {
	NCCoord *c = *it;
	NCRoute *r = dynamic_cast<NCRoute*>(*it);

	if ( c->GetTool() != pCurrentTool )
	{
	    pCurrentTool = c->GetTool();
	    CString s;
	    if ( r == NULL )
		s = "Drill ";
	    else
		s = "Router ";
	    s += FormatNumber( pCurrentTool->Size );
	    s += m_bMetric ? " mm" : " in";
	    gc.ToolChange( pCurrentTool->Id, s );
	}

	CPointF src( c->X, c->Y );
	CPointF dest = m_TransformMatrix.TransformPoint( src );

	if ( r == NULL && m_bDrill )
	{
	    // Drill
	    if ( m_bCannedDrill )
	    {
		gc.Append( "G81 X%s Y%s Z%s R%s F%s", FormatNumber(dest.X), FormatNumber(dest.Y), FormatNumber(m_dDrillDepth), FormatNumber(m_dClearance), FormatNumber(m_dPlungeSpeed) );
	    }
	    else
	    {
		gc.Append( "G00 Z%s", FormatNumber(m_dClearance) );
		gc.Append( "G00 X%s Y%s", FormatNumber(dest.X), FormatNumber(dest.Y) );
		gc.Append( "G01 Z%s F%s", FormatNumber(m_dDrillDepth), FormatNumber(m_dPlungeSpeed) );
		gc.Append( "G00 Z%s", FormatNumber(m_dClearance) );
	    }
	}
	else if ( r != NULL && m_bRoute )
	{ 
	    // route
	    if ( r->Plunged )
	    {
		if ( !bPlunged )
		{
		    gc.Append( "G00 Z%s", FormatNumber(m_dClearance) );
		    gc.Append( "G01 Z%s F%s", FormatNumber(m_dDrillDepth), FormatNumber(m_dPlungeSpeed) );
		    bPlunged = true;
		}
	    }
	    else if ( !r->Plunged )
	    {
		// Move speed
		if ( bPlunged )
		{
		    gc.Append( "G00 Z%s", FormatNumber(m_dClearance) );
		    bPlunged = false;
		}
	    }

	    if ( bPlunged )
		gc.Append( "G01 X%s Y%s F%s", FormatNumber(dest.X), FormatNumber(dest.Y), FormatNumber(m_dRouteSpeed) );
	    else
		gc.Append( "G00 X%s Y%s", FormatNumber(dest.X), FormatNumber(dest.Y) );
	}
    }
    gc.Append( "G00 Z%s", FormatNumber(m_dClearance) );
    gc.ProgramStop();

    char sPath[MAX_PATH];
    GetTempPath( countof(sPath), sPath );

    char sFilename[MAX_PATH];
    GetTempFileName( sPath, "PCB", 0, sFilename );

    CStdioFile outfile( sFilename, CFile::modeCreate | CFile::modeWrite | CFile::shareExclusive );
    outfile.Write( gc.str(), gc.str().GetLength() );
    outfile.Close();

    Mach4::IMyScriptObjectPtr pObj = GetMachPointer();
    if ( pObj != NULL )
    {
	pObj->LoadFile( sFilename );
    }
}




void CPCBDrillDlg::OnBnClickedButtonDelete()
{
    // if a located point is selected, then delete it.
    if ( m_SelectedCoord != NULL )
    {
	LocatedCoordCollection::iterator it;
	for ( it = m_LocatedCoords.begin(); it != m_LocatedCoords.end(); it++ )
	{
	    CLocatedCoord &c = *it;

	    if ( c.m_coord == m_SelectedCoord )
	    {
		if ( MessageBox( "Do you wish to remove the located coordinate mapping?", "Remove coordinate", MB_YESNO ) == IDYES )
		{
		    m_LocatedCoords.erase( it );
		    InvalidateRect( m_rcPCBArea, false );
		}
		break;
	    }
	}

    }
}
