#pragma once

// MachPCBDrill, NC drill/routing program for Mach3
// Copyright (C) 201, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


#include "NCDrillFile.h"
#include "afxwin.h"
#include "afxcmn.h"
#include "Transformation.h"

class CLocatedCoord
{
public:
    double m_x;
    double m_y;
    NCCoord *m_coord;

    CLocatedCoord(NCCoord *oCoord, double dX, double dY)
    {
	m_x = dX;
	m_y = dY;
	m_coord = oCoord;
    }
};


// CPCBDrillDlg dialog
class CPCBDrillDlg : public CDialog
{
    // Construction
public:
    CPCBDrillDlg(CWnd* pParent = NULL);
    virtual ~CPCBDrillDlg();

    // Dialog Data
    enum { IDD = IDD_PCBDRILL_DIALOG };

    // Interface
    void LogMessage( LPCSTR sFormat, va_list args );

protected:
    virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support

private:
    CNCDrillFile *m_drillfile;
    CRect m_rcPCBArea;
    double m_dScale;
    int m_nXOffset, m_nYOffset;
    bool m_bResized;
    NCCoord *m_HighlightCoord;
    NCCoord *m_SelectedCoord;
    NCCoord *m_DrillCoord;
    typedef std::list<CLocatedCoord> LocatedCoordCollection;
    LocatedCoordCollection m_LocatedCoords;
    CPen m_penLocated;
    CPen m_penMove;
    CPen m_penMoveHighlight;
    CPen m_penRoute;
    CPen m_penRouteHighlight;
    CBrush m_brBkgnd;
    CMatrix m_TransformMatrix;
    CPoint m_ptLast;
    CString m_sFileName;
    bool m_bDrill;
    bool m_bRoute;
    CString m_sInitString;

    void ResizeWindow();
    void CalculateScale();
    void LoadConfigFile( LPCSTR sFilename );
    void CalculateTransformation();
    void EnableControls();
    void LoadConfig( MSXML2::IXMLDOMNodePtr pAppConfigNode );
    void Close();
    void SaveConfig();
    void RecomputeScreenCoordinates( int nPadSize );
    void SetConnectState( bool bConnected );
    CString FormatNumber( double d );
    void DoTransform();

    Mach4::IMyScriptObjectPtr GetMachPointer();
    Mach4::IMyScriptObjectPtr m_pScriptObj;
    Mach4::IMach4Ptr m_pMach4;
    bool m_bFlip;
    double m_dAngle;

    // Implementation
protected:
    HICON m_hIcon;
    CStatusBarCtrl m_statusBar;
    CEdit m_txtLog;
    CStatic m_staticIndicator;

    // Generated message map functions
    virtual BOOL OnInitDialog();
    afx_msg void OnPaint();
    afx_msg HCURSOR OnQueryDragIcon();
    DECLARE_MESSAGE_MAP()
    afx_msg void OnFileOpen();
    afx_msg void OnFileExit();
    afx_msg void OnFilePreferences();
    afx_msg void OnGetMinMaxInfo( MINMAXINFO* lpMMI  );
    afx_msg void OnSize( UINT nType, int cx, int cy  );
public:
    afx_msg void OnMouseMove(UINT nFlags, CPoint point);
    afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
    afx_msg void OnBnClickedButtonLocate();
    afx_msg void OnBnClickedButtonGoto();
protected:
    virtual void OnOK();
    virtual void OnCancel();
public:
    CButton m_chkRoute;
    CButton m_chkDrill;
    afx_msg void OnBnClickedCheckRoute();
    afx_msg void OnBnClickedCheckDrill();
    afx_msg void OnBnClickedFlip();
    afx_msg void OnTimer(UINT_PTR nIDEvent);
    CStatic m_lblMachX;
    CStatic m_lblMachY;
    CStatic m_lblMachZ;
    CButton m_btnMM;
    CButton m_btnIN;
    CEdit m_txtClearance;
    CEdit m_txtRouteStep;
    CEdit m_txtPlungeSpeed;
    CEdit m_txtRouteSpeed;
    CEdit m_txtDrillCycle;
    afx_msg void OnBnClickedButtonRotate();
    afx_msg void OnBnClickedButtonFlip();
    double m_dClearance;
    double m_dDrillDepth;
    double m_dRouteStep;
    double m_dPlungeSpeed;
    double m_dRouteSpeed;
    CString m_sCannedCode;
    bool m_bCannedDrill;
    bool m_bMetric;
    bool m_bRouteStep;
    CButton m_chkRouteStep;
    CButton m_chkDrillCycle;
    afx_msg void OnBnClickedCheckDrillCycle();
    afx_msg void OnBnClickedCheckRouteStep();
    afx_msg void OnBnClickedButtonGcode();
    afx_msg void OnBnClickedButtonDelete();
};
