#pragma once

// MachPCBDrill, NC drill/routing program for Mach3
// Copyright (C) 201, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.


class CPointF
{
public:
    double X;
    double Y;

    CPointF()
    {
	X = 0;
	Y = 0;
    }
    CPointF( double x, double y )
    {
	X = x;
	Y = y;
    }
};

class CMatrix
{
public:
    double m[3][3];

    CMatrix()
    {
	memset(m,0,sizeof(m));
    }

    CMatrix( double m11, double m12, double m21, double m22, double offsetX, double offsetY )
    {
	m[0][0] = m11;
	m[1][0] = m12;
	m[2][0] = 0;
	m[0][1] = m21;
	m[1][1] = m22;
	m[2][1] = 0;
	m[0][2] = offsetX;
	m[1][2] = offsetY;
	m[2][2] = 1;
    }
    CPointF TransformPoint( CPointF &src )
    {
	CPointF ret;
	ret.X = m[0][0] * src.X + m[0][1] * src.Y + m[0][2];
	ret.Y = m[1][0] * src.X + m[1][1] * src.Y + m[1][2];
	return ret;
    }
    bool IsIdentity() const
    {
	if ( m[0][0] == 1.0 &&
	    m[1][0] == 0.0 &&
	    m[2][0] == 0.0 &&
	    m[0][1] == 0.0 &&
	    m[1][1] == 1.0 &&
	    m[2][1] == 0.0 &&
	    m[0][2] == 0.0 &&
	    m[1][2] == 0.0 &&
	    m[2][2] == 1.0 )
	    return true;
	else
	    return false;
    }
};

#define T_SIZE	6

class CTransformation
{
public:
    CTransformation(void);
    ~CTransformation(void);
    static CMatrix CTransformation::affineXformCoeffs(CPointF dest[], CPointF src[]);
    static int gaussjordan(double a[T_SIZE][T_SIZE], double b[T_SIZE]);
};
