// MachPCBDrill, NC drill/routing program for Mach3
// Copyright (C) 201, Frank Tkalcevic, www.franksworkshop.com

// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

#include "StdAfx.h"
#include "XmlUtil.h"

CXmlUtil::CXmlUtil(void)
{
}

CXmlUtil::~CXmlUtil(void)
{
}

static bool GetAttributeText( MSXML2::IXMLDOMNodePtr &pNode, LPCTSTR sName, CString &sRet )
{
    MSXML2::IXMLDOMElementPtr pElem = pNode;
    MSXML2::IXMLDOMAttributePtr pAttributeNode = pElem->getAttributeNode( sName );
    if ( pAttributeNode == NULL )
	return false;

    _bstr_t s = pAttributeNode->text;
    sRet = (LPCTSTR)s;
    return true;
}


CString CXmlUtil::GetAttributeString( MSXML2::IXMLDOMNodePtr pNode, LPCTSTR sName, LPCTSTR sDefault )
{
    CString s;
    if ( GetAttributeText(pNode,sName, s) )
	return s;
    return sDefault;
}


int CXmlUtil::GetAttributeInt( MSXML2::IXMLDOMNodePtr pNode, LPCTSTR sName, int nDefault )
{
    CString s;
    if ( GetAttributeText(pNode,sName, s) )
    {
	TCHAR *pEnd;
	s.Trim();
	int nRet = _tcstol( s, &pEnd, 10 );
	if ( pEnd != (LPCTSTR)s )
	    return nRet;
    }
    return nDefault;
}

double CXmlUtil::GetAttributeDouble( MSXML2::IXMLDOMNodePtr pNode, LPCTSTR sName, double dDefault )
{
    CString s;
    if ( GetAttributeText(pNode,sName, s) )
    {
	TCHAR *pEnd;
	s.Trim();
	double dRet = _tcstod( s, &pEnd );
	if ( pEnd != (LPCTSTR)s )
	    return dRet;
    }
    return dDefault;
}

bool CXmlUtil::GetAttributeBool( MSXML2::IXMLDOMNodePtr pNode, LPCTSTR sName, bool bDefault )
{
    CString s;
    if ( GetAttributeText(pNode,sName, s) )
    {
	s.MakeLower();
	if ( s == _T("true") )
	    return true;
	else if ( s == _T("false") )
	    return false;
    }
    return bDefault;
}






void CXmlUtil::AddAttribute( MSXML2::IXMLDOMElementPtr pElem, LPCTSTR sName, double dValue )
{
    CString s;
    s.Format( _T("%g"), dValue );
    pElem->setAttribute( sName, (LPCTSTR)s );
}


void CXmlUtil::AddAttribute( MSXML2::IXMLDOMElementPtr pElem, LPCTSTR sName, int nValue )
{
    CString s;
    s.Format( _T("%d"), nValue );
    pElem->setAttribute( sName, (LPCTSTR)s );
}


void CXmlUtil::AddAttribute( MSXML2::IXMLDOMElementPtr pElem, LPCTSTR sName, GUID &guid )
{
    const int GUID_STRING_LENGTH = 255;
#ifdef UNICODE
    RPC_WSTR pStr;
#else
    RPC_CSTR pStr;
#endif
    int nLen = UuidToString( &guid, &pStr );
    pElem->setAttribute( sName, (LPSTR)pStr );
    RpcStringFree( &pStr );
}

void CXmlUtil::AddAttributeBool( MSXML2::IXMLDOMElementPtr pElem, LPCTSTR sName, bool bValue )
{
    pElem->setAttribute( sName, bValue ? _T("True") : _T("False") );
}

void CXmlUtil::AddAttribute( MSXML2::IXMLDOMElementPtr pElem, LPCTSTR sName, LPCTSTR sValue )
{
    pElem->setAttribute( sName, sValue );
}

std::vector<CString> SplitPath( LPCTSTR szPath )
{
    std::vector<CString> s;

    CString sToken;
    CString sPath(szPath);
    int n = 0;
    sToken = sPath.Tokenize( _T("/"), n );
    while ( n >= 0 )
    {
	s.push_back( sToken );
	sToken = sPath.Tokenize( _T("/"), n );
    }
    return s;
}

void CXmlUtil::ReplaceNode( LPCTSTR sPath, MSXML2::IXMLDOMNodePtr pNewNode)
{
    MSXML2::IXMLDOMDocumentPtr pDoc = pNewNode->ownerDocument;

    MSXML2::IXMLDOMNodePtr pOldNode = pDoc->selectSingleNode( sPath );
    if ( pOldNode != NULL )
    {
	MSXML2::IXMLDOMNodePtr pParent = pOldNode->parentNode;
	pParent->replaceChild( pNewNode, pOldNode );
    }
    else
    {
	pOldNode = pDoc;

	std::vector<CString> s = SplitPath( sPath );
	std::vector<CString>::iterator it;
	for  ( it = s.begin(); it != s.end(); it++ )
	{
	    CString &sNode = *it;
	    MSXML2::IXMLDOMNodePtr pNode = pOldNode->selectSingleNode( (LPCTSTR)sNode );
	    if ( pNode == NULL )
	    {
		pNode = pDoc->createElement( (LPCTSTR)sNode );
		pOldNode->appendChild( pNode );
	    }
	    pOldNode = pNode;
	}
	MSXML2::IXMLDOMNodePtr pParent = pOldNode->parentNode;
	pParent->replaceChild( pNewNode, pOldNode );
    }
}
